# Hestia Calculation Engine

Hestia's set of modules for creating structured data models from LCA observations and evaluating biogeochemical aspects of specific farming cycles

## Install

1. Install the module:
```bash
pip install hestia_earth.calculation
```

2. Download the latest data:
```bash
curl https://gitlab.com/hestia-earth/hestia-calculation-engine/-/raw/master/scripts/download_data.sh?inline=false -o download_data.sh && chmod +x download_data.sh
# pip default install directory is /usr/local/lib/python<version>/site-packages
./download_data.sh <API_URL> <SEARCH_URL> <pip install directory>
```

To use the production-ready version of the data, please use `API_URL=https://api.hestia.earth` and `SEARCH_URL=https://search.hestia.earth`. For development version, please contact us at <community@hestia.earth>

## Usage

```python
from hestia_earth.calculation import CalculationEngine
# Create CalculationEngine object
engine = CalculationEngine('url')
# ingest data ('cycle', 'site', 'assessments' are keyword arguments)
engine.import_data(cycle=ingested_cycle, site=ingested_site, assessments=ingested_assessments)
# do the calculations
engine.run()
# retrieve the calculated data
engine.export_data()
```


### Recalculating a Cycle

If only a cycle is provided it only recalculates the cycle,

```python
cycle = {"@type": "Cycle", ...}
engine.import_data(cycle=cycle)
engine.run()
data = engine.export_data()
# data = {"@type": "Cycle", ...}  (recalculated cycle)
```

and the exported object is the recalculated cycle. In this case there are no assessments calculated.


### Recalculating a Cycle with a Site


If a cycle and an a site are provided it recalculates everything with the data of the cycle and the site,

```python
cycle = {"@type": "Cycle", ...}
site = {"@type": "Site", ...}
engine.import_data(cycle=cycle, site=site)
engine.run()
data = engine.export_data()
# data = {"@type": "Cycle", ...}  (recalculated cycle)
```

and the exported object is again a recalculated cycle. In this case there are no assessments calculated.


### Recalculating ImpactAssessment


If only assessment is provided then the engine recalculates the assessments with the cycle specified inside every assessment. 

```python
assessment = [{"@type": "ImpactAssessment", "name": "Assessment_1" ...}, {"@type": "ImpactAssessment", "name": "Assessment_2" ...}, ...]
engine.import_data(assessment=assessment)
engine.run()
data = engine.export_data()
# data = [{"@type": "ImpactAssessment", "name": "Assessment_1" ...}, {"@type": "ImpactAssessment", "name": "Assessment_2" ...}, {"@type": "ImpactAssessment", "name": "Assessment_3", ...] (recalculated and updated assessments)
```
 
The engine then does the calculations and updates the assessments for the values that are different if they exist or creating new assessments if they do not exist. The exported object is a list with the recalculated assessments.


### Recalculating ImpactAssessment with Cycle and Site


If the three arguments are provided (cycle, site, and assessment) then the calculations are done with the cycle and site especifically indicated and not taking into account the one inside every assessment. 

```python
cycle = {"@type": "Cycle", ...}
site = {"@type": "Site", ...}
assessment = [{"@type": "ImpactAssessment", "name": "Assessment_1" ...}, {"@type": "ImpactAssessment", "name": "Assessment_2" ...}, ...]
engine.import_data(cycle=cycle, site=site, assessment=assessment)
engine.run()
data = engine.export_data()
# data = [{"@type": "ImpactAssessment", "name": "Assessment_1" ...}, {"@type": "ImpactAssessment", "name": "Assessment_2" ...}, ...] (recalculated and updated assessments)
```
 
The engine then does the calculations and updates the assessments for the values that are different if they exist or creating new assessments if they do not exist. The exported object is a list with the recalculated assessments.




