import importlib
import inspect
import concurrent.futures
from functools import partial
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.abstract_hierarchy import Hierarchy
from hestia_earth.calculation.assessment_hierarchies import Assessments_hierarchy


class CalculationEngine(Engine):
    """Engine class. This is the class that runs all the models for all emissions and impacts."""
    def __init__(self, baseUrl):
        """Constructor method.
            Arguments:
                None
            Returns instance."""

        super().__init__()
        Hierarchy.context = baseUrl

        self.assessment = Assessments_hierarchy()

        order = ["co2ToAirLimeHydrolysis",
                 "co2ToAirUreaHydrolysis",
                 "co2ToAirFuelCombustion",
                 "n2OToAirInorganicFertilizerDirect",
                 "n2OToAirOrganicFertilizerDirect",
                 "n2OToAirCropResidueBurningDirect",
                 "n2OToAirExcretaDirect",
                 "n2OToAirCropResidueDecompositionDirect",
                 "nh3ToAirCropResidueBurning",
                 "nh3ToAirCropResidueDecomposition",
                 "nh3ToAirInorganicFertilizer",
                 "nh3ToAirOrganicFertilizer",
                 "no3ToGroundwaterInorganicFertilizer",
                 "no3ToGroundwaterOrganicFertilizer",
                 "no3ToGroundwaterExcreta",
                 "no3ToGroundwaterCropResidueDecomposition",
                 "noxToAirCropResidueDecomposition",
                 "noxToAirInorganicFertilizer",
                 "noxToAirOrganicFertilizer",
                 "noxToAirExcreta",
                 "noxToAirCropResidueBurning",
                 "ch4ToAirCropResidueBurning",
                 "co2ToAirAllOrigins",
                 "so2ToAirAllOrigins",
                 "noxToAirAllOrigins",
                 "nh3ToAirAllOrigins",
                 "n2OToAirAllOriginsDirect",
                 "ch4ToAirAllOriginsFossil",
                 "ch4ToAirAllOriginsNonFossil",
                 "pErosionAllOrigins",
                 "pToGroundwaterAllInputs",
                 "pToSurfacewaterAllInputs",
                 "pToDrainageWaterAllInputs",
                 "n2OToAirInorganicFertilizerIndirect",
                 "n2OToAirOrganicFertilizerIndirect",
                 "n2OToAirExcretaIndirect",
                 "n2OToAirCropResidueDecompositionIndirect"
                 ]

        func = partial(self.load_hierarchy, directory="emissions")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            self.hierarchies = list(executor.map(func, order))

    def load_hierarchy(self, name, directory=None):
        module = importlib.import_module(f"hestia_earth.calculation.{directory}.{name}")
        obj = [tuple[1]() for tuple in inspect.getmembers(module, inspect.isclass) if
               tuple[0].split("_")[0] == "Hierarchy"]
        return obj[0]
