from hestia_earth.calculation.abstract_model import Model
from statistics import mean


class LandOccupation(Model):
    def calculate(self, cycle, product):

        # If statement to define which products are run (for example, here, run for those with EVS > 0)
        if product.get('economicValueShare', 0) > 0 and product['value'] != [] and \
                cycle['functionalUnitMeasure'] == '1 ha':

            # calculate amount of land per kg of product for temporary and permanent crops
            # TODO: Add a catch here, do the following if product is a temporary or permanent crop,
            #  different model for orchard crops

            # set cycle level variables
            temp_seed = cycle["inputs"]["seed"]
            temp_cycleDuration = cycle["cycleDuration"]
            temp_rotationDuration = cycle["practices"]["rotationDuration"]
            temp_longFallowPeriod = cycle["practices"]["longFallowPeriod"]

            # 1) Estimate land area (m2) needed to produce 1kg of product (10000/yield_Kg)
            product_amount = float(mean(product['value']))

            land_use_1 = 10000 / product_amount

            # 2) multiply by yield+seed as a proportion of yield (to account for land use needed for the seed)
            if temp_seed != {}:
                land_use_2 = land_use_1 * ((float(cycle["inputs"]['seed']['value'][0]) +
                                            product_amount) / product_amount)
            else:
                land_use_2 = land_use_1

            # 3) Account for crop duration (multiple crops on a given field in a given year
            # TODO: Add an example fixture containing cycleDuration to test this
            if temp_cycleDuration != {}:
                land_use_3 = land_use_2 * (float(cycle["cycleDuration"]) / 365)
                # TODO: need to account for alternativeStartDate as this can override default cycle duration
            else:
                land_use_3 = land_use_2

            # 4) Account for fallow period in crop production
            # TODO: Add an example fixture containing rotationDuration and longFallowPeriod to test this
            if temp_rotationDuration != {} and temp_longFallowPeriod != {}:
                cultivated_duration = float(cycle["practices"]["rotationDuration"]['value'][0]) - \
                                      float(cycle["practices"]["longFallowPeriod"]['value'][0])
                land_use_4 = land_use_3 * (float(cycle["practices"]["rotationDuration"]['value'][0]) /
                                           cultivated_duration)
            else:
                land_use_4 = land_use_3

            # TODO: account for numberOfCycles (for example the wheat example fixture over 41 years).
            #  Discuss with JP + Juan

            # create emissions resource use node
            emissionsResourceUses = {}
            emissionsResourceUses['landOccupation'] = land_use_4

            return emissionsResourceUses
