from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.data.constants.ch4 import DRY_MATTER_FACTOR_TO_CH4
from hestia_earth.calculation.data.constants.n2o import DRY_MATTER_FACTOR_TO_NO2
from hestia_earth.calculation.data.constants.nh3 import DRY_MATTER_FACTOR_TO_NH3
from hestia_earth.calculation.data.constants.nox import DRY_MATTER_FACTOR_TO_NOX


class CH4N2ONH3AndNOXToAirCropResidueBurningAkagiEtAl2011AndIpcc2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Instantiate variables
        self.ch4ToAirCropResidueBurning = None
        self.n2OToAirCropResidueBurningDirect = None
        self.nh3ToAirCropResidueBurning = None
        self.noxToAirCropResidueBurning = None

        # Define model coefficients
        self.drymatter_to_ch4 = DRY_MATTER_FACTOR_TO_CH4
        self.drymatter_to_n2o = DRY_MATTER_FACTOR_TO_NO2
        self.drymatter_to_nh3 = DRY_MATTER_FACTOR_TO_NH3
        self.drymatter_to_nox = DRY_MATTER_FACTOR_TO_NOX

    def calculate_ch4ToAirCropResidueBurning(self):

        self.ch4ToAirCropResidueBurning = self.burnt * self.drymatter_to_ch4 *\
                                          (1 - self.left_on_field_r) * self.residue_burnt_r

        return self.ch4ToAirCropResidueBurning

    def calculate_n2OToAirCropResidueBurningDirect(self):

        self.n2OToAirCropResidueBurningDirect = self.burnt * self.drymatter_to_n2o *\
                                          (1 - self.left_on_field_r) * self.residue_burnt_r

        return self.n2OToAirCropResidueBurningDirect

    def calculate_nh3ToAirCropResidueBurning(self):

        self.nh3ToAirCropResidueBurning = self.burnt * self.drymatter_to_nh3 *\
                                          (1 - self.left_on_field_r) * self.residue_burnt_r

        return self.nh3ToAirCropResidueBurning

    def calculate_noxToAirCropResidueBurning(self):

        self.noxToAirCropResidueBurning = self.burnt * self.drymatter_to_nox *\
                                          (1 - self.left_on_field_r) * self.residue_burnt_r

        return self.noxToAirCropResidueBurning

    def get_burnt(self, cycle):

        filter_list = ['aboveGroundCropResidueDryMatter', 'belowGroundCropResidueDryMatter']

        products = cycle["products"].values()
        burnt = sum([sum(product['value']) for product in products if product['term']['@id'] in filter_list and
                     'destination' in product and 'burning' == product['destination']['@id']])

        return burnt

    def check_ch4ToAirCropResidueBurning(self, cycle):
        # Check that we have all the inputs
        self.left_on_field_r = float(cycle['practices']['residueLeftOnField']['value'])\
            if cycle['practices']['residueLeftOnField']['value'] != {} else {}
        self.residue_burnt_r = float(cycle['practices']['residueBurnt']['value'])\
            if cycle['practices']['residueBurnt']['value'] != {} else {}
        self.burnt = self.get_burnt(cycle)

        return self.left_on_field_r != {} and self.residue_burnt_r != {}

    def check_n2OToAirCropResidueBurningDirect(self, cycle):
        # Check that we have all the inputs
        self.left_on_field_r = float(cycle['practices']['residueLeftOnField']['value'])\
            if cycle['practices']['residueLeftOnField']['value'] != {} else {}
        self.residue_burnt_r = float(cycle['practices']['residueBurnt']['value'])\
            if cycle['practices']['residueBurnt']['value'] != {} else {}
        self.burnt = self.get_burnt(cycle)

        return self.left_on_field_r != {} and self.residue_burnt_r != {}

    def check_nh3ToAirCropResidueBurning(self, cycle):
        # Check that we have all the inputs
        self.left_on_field_r = float(cycle['practices']['residueLeftOnField']['value'])\
            if cycle['practices']['residueLeftOnField']['value'] != {} else {}
        self.residue_burnt_r = float(cycle['practices']['residueBurnt']['value'])\
            if cycle['practices']['residueBurnt']['value'] != {} else {}
        self.burnt = self.get_burnt(cycle)

        return self.left_on_field_r != {} and self.residue_burnt_r != {}

    def check_noxToAirCropResidueBurning(self, cycle):
        # Check that we have all the inputs
        self.left_on_field_r = float(cycle['practices']['residueLeftOnField']['value'])\
            if cycle['practices']['residueLeftOnField']['value'] != {} else {}
        self.residue_burnt_r = float(cycle['practices']['residueBurnt']['value'])\
            if cycle['practices']['residueBurnt']['value'] != {} else {}
        self.burnt = self.get_burnt(cycle)

        return self.left_on_field_r != {} and self.residue_burnt_r != {}
