from hestia_earth.calculation.abstract_model import Model


class CO2ToAirLimeAndUreaIpcc2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.total_lime = None
        self.total_dolomiticLime = None

        self.ureaAsN = None
        self.ureaAmmoniumNitrateAsN = None
        self.ureaAmmoniumSulphateAsN = None

        # Instantiate variables
        self.co2ToAirLimeHydrolysis = None
        self.co2ToAirUreaHydrolysis = None

        # Define model coeffients
        self.co2_lime_factor = 0.43966033966033968
        self.co2_dolomite_factor = 0.47629870129870133
        self.co2_urea_factor = 1.5710002141786252
        self.conv_fert_UAN_Urea = 0.5
        self.conv_fert_UAS_Urea = 0.8

    def calculate_co2ToAirLimeHydrolysis(self):

        # Calculate CO2 emissions

        self.co2ToAirLimeHydrolysis = sum(self.total_lime) * self.co2_lime_factor + \
                                      sum(self.total_dolomiticLime) * self.co2_dolomite_factor

        return self.co2ToAirLimeHydrolysis

    def calculate_co2ToAirUreaHydrolysis(self):

        # Calculate CO2 emissions

        self.co2ToAirUreaHydrolysis = (sum(self.ureaAsN) + sum(self.ureaAmmoniumSulphateAsN) * self.conv_fert_UAS_Urea +
                                       sum(self.ureaAmmoniumNitrateAsN) * self.conv_fert_UAN_Urea) *\
                                       self.co2_urea_factor

        return self.co2ToAirUreaHydrolysis

    def check_co2ToAirLimeHydrolysis(self, cycle):

        # Get inputs

        inputs = cycle['inputs'].evalues()

        soilAmends = [input for input in inputs
                      if input['term']['termType'] == "soilAmendment"]

        self.total_lime = [sum(input['value']) for input in soilAmends
                           if input['term']['@id'] == "lime" or
                           input['term']['@id'] == "aragonite" or
                           input['term']['@id'] == "agriculturalLimestone" or
                           input['term']['@id'] == "hydratedLime" or
                           input['term']['@id'] == "burntLime"]

        # Calculate total dolomitelime input

        self.total_dolomiticLime = [sum(input['value'])for input in soilAmends
                                    if input['term']['@id'] == "dolomiticLime"]

        return self.total_lime != [] or self.total_dolomiticLime != []

    def check_co2ToAirUreaHydrolysis(self, cycle):

        # Get inputs

        inputs = cycle['inputs'].evalues()

        self.ureaAsN = [sum(input['value']) for input in inputs if input['term']['termType'] == "inorganicFertilizer"
                        and input['term']['@id'] == "ureaAsN"]

        self.ureaAmmoniumNitrateAsN = [sum(input['value']) for input in inputs
                                       if input['term']['termType'] == "inorganicFertilizer" and
                                       input['term']['@id'] == "ureaAmmoniumNitrateAsN"]

        self.ureaAmmoniumSulphateAsN = [sum(input['value']) for input in inputs
                                        if input['term']['termType'] == "inorganicFertilizer" and
                                        input['term']['@id'] == "ureaAmmoniumSulphateAsN"]

        return self.ureaAsN != [] or self.ureaAmmoniumNitrateAsN != [] or self.ureaAmmoniumSulphateAsN != []
