from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.utils import summation


class N2OToAirCropResidueBurningAkagiEtAl2011AndIpcc2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.dry_matter = None
        self.left_on_field_r = None
        self.residue_burnt_r = None

        # Instantiate variables
        self.n2OToAirCropResidueBurningDirect = None

        # Define model coeffients
        self.conv_dm_n2o = 0.00007     # TODO HLT

    def calculate_n2OToAirCropResidueBurningDirect(self):
        # Calculate total N2O emission

        self.n2OToAirCropResidueBurningDirect = self.dry_matter * self.conv_dm_n2o * (1 - self.left_on_field_r) *\
                                                self.residue_burnt_r

        return self.n2OToAirCropResidueBurningDirect

    def check_n2OToAirCropResidueBurningDirect(self, cycle):
        # Check that we have all the inputs

        filter_list = ['aboveGroundCropResidueDryMatter', 'belowGroundCropResidueDryMatter']

        products = cycle['products'].evalues()

        self.dry_matter = summation([sum(product['value']) for product in products
                                     if product['term']['@id'] in filter_list and 'destination' in product and
                                     'burning' == product['destination']['@id']])
        self.left_on_field_r = cycle['practices']['residueLeftOnField']['value']
        self.residue_burnt_r = cycle['practices']['residueBurnt']['value']

        return self.dry_matter != {} and self.left_on_field_r != {} and self.residue_burnt_r != {}
