import numpy as np
from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.data.constants.nh3 import NH3_TAN_FACTOR


class NH3ToAirOrganicFertilizerWebbEtAl2012AndSintermannEtAl2012(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.lqd_slurry_sluge = None
        self.solid = None
        self.compost = None
        self.greenManure = None

        # Instantiate variables
        self.nh3ToAirOrganicFertilizer = None

        # Define model coeffients
        self.eafs_lookup = NH3_TAN_FACTOR

    def calculate_nh3ToAirOrganicFertilizer(self):
        # Calculate total nh3 emissions

        eafs = self.eafs_lookup.values()

        n_inputs = np.array([self.lqd_slurry_sluge, self.solid, self.compost, self.greenManure])

        self.nh3ToAirOrganicFertilizer = sum([x * y for x, y in zip(n_inputs, eafs)])

        return self.nh3ToAirOrganicFertilizer

    def access_lqd_slurry_sluge(self, cycle):

        slurryAndSludgeAsN = cycle['inputs']['slurryAndSludgeAsN']['value']
        cattleLiquidManureAsN = cycle['inputs']['cattleLiquidManureAsN']['value']
        pigsLiquidManureAsN = cycle['inputs']['pigsLiquidManureAsN']['value']

        lqd_slurry_sluge = [slurryAndSludgeAsN, cattleLiquidManureAsN, pigsLiquidManureAsN]

        if all(v == {} for v in lqd_slurry_sluge):
            self.lqd_slurry_sluge = None
        else:
            self.lqd_slurry_sluge = sum([sum(i) for i in filter(None, lqd_slurry_sluge)])

    def access_solid(self, cycle):

        non_solid = ['cattleUrineAsN', 'compostAsN', 'greenManureAsN', 'oliveMillWastewaterAsN',
                     'palmOilMillEffluentAsN', 'pigsLiquidManureAsN', 'sewageSludgeAsN', 'slurryAndSludgeAsN']

        inputs = list(cycle['inputs'].evalues())

        orgN_filter = [sum(input['value']) if 'units' in input['term'] and input['term']['@id'] not in non_solid and
                       input['term']['units'] == 'kg N' and
                       input['term']['termType'] == 'organicFertilizer'
                       else 0 for input in inputs]

        self.solid = None if all(v is None for v in orgN_filter) else\
            sum(filter(None, orgN_filter))

    def access_compost(self, cycle):

        compostAsN = cycle['inputs']['compostAsN']['value']
        mushroomCompostAsN = cycle['inputs']['mushroomCompostAsN']['value']

        compost = [compostAsN, mushroomCompostAsN]

        if all(v == {} for v in compost):
            self.compost = None
        else:
            self.compost = sum([sum(i) for i in filter(None, compost)])

    def access_greenManure(self, cycle):

        greenManure_array = cycle['inputs']['greenManureAsN']['value']
        self.greenManure = sum(greenManure_array) if greenManure_array != {} else None

    def check_nh3ToAirOrganicFertilizer(self, cycle):
        # Check that we have all the inputs

        self.access_lqd_slurry_sluge(cycle)
        self.access_solid(cycle)
        self.access_compost(cycle)
        self.access_greenManure(cycle)

        return self.lqd_slurry_sluge is not None and self.solid is not None and\
            self.compost is not None and self.greenManure is not None
