from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.utils import summation


class NOXToAirCropResidueBurningAkagiEtAl2011AndIpcc2006(Model):
    def __init__(self):
        # Define model tier
        self.tier = 1

        # Define model requirements
        self.dry_matter = None
        self.left_on_field_r = None
        self.residue_burnt_r = None

        # Instantiate variables
        self.noxToAirCropResidueDecomposition = None

        # Define model coeffients
        self.conv_dm_nox = 3.11/1000     # TODO HLT

    def calculate_noxToAirCropResidueDecomposition(self):
        # Calculate total NOX emission

        self.noxToAirCropResidueDecomposition = self.dry_matter * self.conv_dm_nox * (1 - self.left_on_field_r) *\
                                                self.residue_burnt_r

        return self.noxToAirCropResidueDecomposition

    def check_noxToAirCropResidueDecomposition(self, cycle):
        # Check that we have all the inputs

        filter_list = ['aboveGroundCropResidueDryMatter', 'belowGroundCropResidueDryMatter']

        products = cycle['products'].evalues()

        self.dry_matter = summation([sum(product['value']) for product in products
                                     if product['term']['@id'] in filter_list and 'destination' in product and
                                     'burning' == product['destination']['@id']])
        self.left_on_field_r = cycle['practices']['residueLeftOnField']['value']
        self.residue_burnt_r = cycle['practices']['residueBurnt']['value']

        return self.dry_matter != {} and self.left_on_field_r != {} and self.residue_burnt_r != {}
