import numpy as np
from hestia_earth.calculation.abstract_model import Model
from hestia_earth.utils.lookup import download_lookup


class NOXToAirFertilizerAndExcretaStehfestBouwman2006Tier1(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.n_inputs = None
        self.n_outputs = None
        self.country = None

        # Instantiate variables
        self.noxToAirAllOrigins = None

        # Define model coeffients
        self.nox_conv_factors = {elem[0]: np.float64(elem[2]) for elem in download_lookup('region.csv')
                                 if elem[2] != '-' and elem[2] != ""}

    def calculate_noxToAirAllOrigins(self):
        # Calculate total NOX emission

        self.noxToAirAllOrigins = self.nox_conv_factors[self.country] * (self.n_inputs + self.n_outputs)

        return self.noxToAirAllOrigins

    def check_noxToAirAllOrigins(self, cycle):
        # Check that we have all the inputs

        inputs = cycle['inputs'].evalues()
        self.n_inputs = sum([sum(input['value']) if 'units' in input['term'] and input['term']['units'] == 'kg N' else
                             0 for input in inputs])

        products = cycle['products'].evalues()
        ab_gr = sum([sum(product['value']) for product in products
                     if product['term']['@id'] == 'aboveGroundCropResidueDryMatter'])
        bl_gr = sum([sum(product['value']) for product in products
                     if product['term']['@id'] == 'belowGroundCropResidueDryMatter'])
        ab_gr_nc = cycle['products']['aboveGroundCropResidueDryMatter']['properties']['nitrogenContent']['value']
        bl_gr_nc = cycle['products']['belowGroundCropResidueDryMatter']['properties']['nitrogenContent']['value']
        self.n_outputs = ab_gr * (float(ab_gr_nc) if ab_gr_nc != {} else 0) + \
            bl_gr * (float(bl_gr_nc) if bl_gr_nc != {} else 0)

        self.country = cycle['site']['country']['@id']

        return (self.n_inputs != {} or self.n_outputs != {}) and len(inputs) > 0 and\
            self.country in self.nox_conv_factors
