from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.utils import most_relevant_measurement, summation


class PToDrainageWaterSalcaPrasuhn2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.orgP = None
        self.drainageClass = None
        self.pToSurfacewaterAllInputs = None

        # Instantiate variables
        self.pToDrainageWaterAllInputs = None

    def calculate_pToDrainageWaterAllInputs(self):

        if self.pToSurfacewaterAllInputs is not None:
            self.pToDrainageWaterAllInputs =\
                0.07 * (1 + 0 if self.orgP == 0 else self.orgP * 0.2/80 * (0 if self.lqd_slurry_sluge is None
                                                                           else self.lqd_slurry_sluge)) * \
                (6 if self.drainageClass > 3 else 0)
        else:
            self.pToDrainageWaterAllInputs = None

        return self.pToDrainageWaterAllInputs

    def check_pToDrainageWaterAllInputs(self, cycle):

        inputs = cycle['inputs'].evalues()

        self.orgP = summation([sum(input['value']) if 'units' in input['term'] and
                                                      input['term']['units'] == 'kg P2O5' and
                                                      input['term']['termType'] == 'organicFertilizer'
                               else 0 for input in inputs])
        self.drainageClass = most_relevant_measurement(cycle['site']['measurements']['drainageClass'], cycle['endDate'])
        pToSurfacewaterAllInputs_data = cycle['emissions']['pToSurfacewaterAllInputs']['value']
        self.pToSurfacewaterAllInputs = pToSurfacewaterAllInputs_data[0] if pToSurfacewaterAllInputs_data != {}\
            else None

        return self.orgP != {} and pToSurfacewaterAllInputs_data != {} and self.drainageClass != {}
