from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.utils import most_relevant_measurement, summation


class PToGroundwaterSalcaPrasuhn2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.orgP = None
        self.drainageClass = None

        # Instantiate variables
        self.pToGroundwaterAllInputs = None

    def get_lqd_slurry_sluge(self, cycle):

        slurryAndSludgeAsN = cycle['inputs']['slurryAndSludgeAsN']['value']
        cattleLiquidManureAsN = cycle['inputs']['cattleLiquidManureAsN']['value']
        pigsLiquidManureAsN = cycle['inputs']['pigsLiquidManureAsN']['value']

        lqd_slurry_sluge = [slurryAndSludgeAsN, cattleLiquidManureAsN, pigsLiquidManureAsN]

        if all(v == {} for v in lqd_slurry_sluge):
            lqd_slurry_sluge = None
        else:
            lqd_slurry_sluge = sum([sum(i) for i in filter(None, lqd_slurry_sluge)])

        return lqd_slurry_sluge

    def calculate_pToGroundwaterAllInputs(self):

        if self.orgP is not None:
            self.pToGroundwaterAllInputs = \
                0.07 * (1 + 0 if self.orgP == 0 else self.orgP * 0.2/80 * (0 if self.lqd_slurry_sluge is None
                                                                           else self.lqd_slurry_sluge)) * \
                (0 if self.drainageClass > 3 else 1)
        else:
            self.pToGroundwaterAllInputs = None

        return self.pToGroundwaterAllInputs

    def check_pToGroundwaterAllInputs(self, cycle):

        inputs = cycle['inputs'].evalues()

        self.orgP = summation([sum(input['value']) if 'units' in input['term'] and
                                                      input['term']['units'] == 'kg P2O5' and
                                                      input['term']['termType'] == 'organicFertilizer'
                               else 0 for input in inputs])

        self.drainageClass = most_relevant_measurement(cycle['site']['measurements']['drainageClass'], cycle['endDate'])
        self.lqd_slurry_sluge = self.get_lqd_slurry_sluge(cycle)

        return self.orgP != {} and self.drainageClass != {}
