from hestia_earth.calculation.abstract_model import Model
from hestia_earth.calculation.utils import most_relevant_measurement, summation


class PToSurfacewaterSalcaPrasuhn2006(Model):
    def __init__(self):

        # Define model tier
        self.tier = 1

        # Define model requirements
        self.inorgP = None
        self.orgP = None
        self.excrP = None
        self.slope = None

        # Instantiate variables
        self.pToDrainageWaterAllInputs = None

    def get_lqd_slurry_sluge(self, cycle):

        slurryAndSludgeAsN = cycle['inputs']['slurryAndSludgeAsN']['value']
        cattleLiquidManureAsN = cycle['inputs']['cattleLiquidManureAsN']['value']
        pigsLiquidManureAsN = cycle['inputs']['pigsLiquidManureAsN']['value']

        lqd_slurry_sluge = [slurryAndSludgeAsN, cattleLiquidManureAsN, pigsLiquidManureAsN]

        if all(v == {} for v in lqd_slurry_sluge):
            lqd_slurry_sluge = None
        else:
            lqd_slurry_sluge = sum([sum(i) for i in filter(None, lqd_slurry_sluge)])

        return lqd_slurry_sluge

    def get_solid(self, cycle):

        non_solid = ['cattleUrineAsN', 'compostAsN', 'greenManureAsN', 'oliveMillWastewaterAsN',
                     'palmOilMillEffluentAsN', 'pigsLiquidManureAsN', 'sewageSludgeAsN', 'slurryAndSludgeAsN']

        inputs = list(cycle['inputs'].evalues())

        orgN_filter = [sum(input['value']) if 'units' in input['term'] and input['term']['@id'] not in non_solid and
                       input['term']['units'] == 'kg N' and
                       input['term']['termType'] == 'organicFertilizer'
                       else None for input in inputs]

        solid = None if all(v is None for v in orgN_filter) else\
            sum(filter(None, orgN_filter))

        return solid

    def get_compost(self, cycle):

        compostAsN = cycle['inputs']['compostAsN']['value']
        mushroomCompostAsN = cycle['inputs']['mushroomCompostAsN']['value']

        compost = [compostAsN, mushroomCompostAsN]

        if all(v == {} for v in compost):
            compost = None
        else:
            compost = sum([sum(i) for i in filter(None, compost)])

        return compost

    def get_greenManure(self, cycle):

        greenManure_array = cycle['inputs']['greenManureAsN']['value']
        greenManure = sum(greenManure_array) if greenManure_array != {} else None

        return greenManure

    def calculate_pToSurfacewaterAllInputs(self):

        self.pToSurfacewaterAllInputs = \
            (0 if self.slope < 0.03 else 1) * (1 + self.inorgP * 0.2/80 + self.orgP * 0.7/80 *
                                               (self.lqd_slurry_sluge if self.lqd_slurry_sluge is not None else 0) +
                                               (self.orgP * self.livestockfert + self.excrP) * 0.4/80)

        return self.pToSurfacewaterAllInputs

    def check_pToSurfacewaterAllInputs(self, cycle):

        inputs = cycle['inputs'].evalues()

        inorgP_filter = [sum(input['value']) if 'units' in input['term'] and input['term']['units'] == 'kg P2O5' and
                         input['term']['termType'] == 'inorganicFertilizer'
                         else None for input in inputs]

        orgP_filter = [sum(input['value']) if 'units' in input['term'] and input['term']['units'] == 'kg P2O5' and
                                              input['term']['termType'] == 'organicFertilizer'
                       else None for input in inputs]

        products = cycle['products'].evalues()

        excrP_filter = [sum(product['value']) if 'units' in product['term'] and
                                                 product['term']['units'] == 'kg P2O5' and
                                                 product['term']['termType'] == 'animalProduct'
                        else None for product in products]

        self.inorgP = summation([element for element in inorgP_filter if element is not None])
        self.orgP = summation([element for element in orgP_filter if element is not None])
        self.excrP = summation([element for element in excrP_filter if element is not None])
        self.slope = most_relevant_measurement(cycle['site']['measurements']['slope'], cycle['endDate'])

        self.lqd_slurry_sluge = self.get_lqd_slurry_sluge(cycle)
        self.solid = self.get_solid(cycle)
        self.compost = self.get_compost(cycle)
        self.greenManure = self.get_greenManure(cycle)

        raw_livestockfert = [self.lqd_slurry_sluge, self.solid, self.compost, self.greenManure]
        self.livestockfert = sum([element for element in raw_livestockfert if element is not None])

        return self.inorgP != {} and self.orgP != {} and self.excrP != {} and self.slope != {}
