from collections import OrderedDict
from statistics import mean
from dateutil import parser

MAX_DEPTH = 1000
OLDEST_DATE = '1800'


def average(data):

    return mean(data) if isinstance(data, list) else {}


def most_recents(measurements, cycle_date):
    dates = list(set([abs((parser.isoparse(measurement['endDate']) - cycle_date).days) if 'endDate' in measurement
                      else abs((parser.isoparse(OLDEST_DATE) - cycle_date).days) for measurement in measurements]))
    dates.sort()
    orderedmeasurements = [[] for i in range(len(dates))]
    for measurement in measurements:
        date = abs((parser.isoparse(measurement['endDate']) - cycle_date).days) \
            if 'endDate' in measurement else abs((parser.isoparse(OLDEST_DATE) - cycle_date).days)
        date_index = dates.index(date)
        orderedmeasurements[date_index].append(measurement)

    return orderedmeasurements[0] if len(orderedmeasurements) > 0 else []


def parse_methodTier(value: str):
    # TODO: methodTier can be 'background' / 'measured', code won't work with these
    return [int(value) for value in value.split() if value.isdigit()][0]


def most_relevant_measurement(measurement, date):
    cycle_date = parser.isoparse(date)
    if isinstance(measurement, EngineDict):
        return average(measurement['value'])
    elif len(measurement) > 0:
        most_recent_measurements = most_recents(measurement, cycle_date)
        shallowest_measurements = shallowest_measurement(most_recent_measurements)
        return average(shallowest_measurements[0]['value'])


def shallowest_measurement(measurements):
    depths = list(set([measurement['depthUpper'] if 'depthUpper' in measurement else MAX_DEPTH
                       for measurement in measurements]))
    depths.sort()
    orderedmeasurements = [[] for i in range(len(depths))]
    for measurement in measurements:
        depth = measurement['depthUpper'] if 'depthUpper' in measurement else MAX_DEPTH
        depth_index = depths.index(depth)
        orderedmeasurements[depth_index].append(measurement)

    return orderedmeasurements[0] if len(orderedmeasurements) > 0 else []


def summation(data):

    return sum(data) if isinstance(data, list) and len(data) > 0 else {}


class EngineDict(OrderedDict):
    """EngineDict class. This class creates dictionaries that do not throw error exceptions when we try to access keys
    that are not in the dictionary. This class also extends dictionaries in order to covert to list EngineDict
    itself."""
    def __missing__(self, key):
        """Method that returns an empty dictionary when a key is not fund.
            Arguments:
                key: key of a dictionary
            Returns empty dictionary."""

        value = type(self)()
        return value

    def evalues(self):
        """Method to generate an EngineList from an EngineDict.
            Arguments:
                None
            Returns EngineList."""

        values = EngineList()
        for element in self.__iter__():
            if isinstance(self.__getitem__(element), list):
                values.extend(self.__getitem__(element))
            else:
                values.append(self.__getitem__(element))

        return values


class EngineList(list):
    """EngineList class. This class also extends lists in order to covert them easily to EngineDict, that is the data
    structure used for the Engine representation."""
    def to_edict(self):
        """Method to generate an EngineDict from an EngineList.
            Arguments:
                None
            Returns EngineDict."""

        items = EngineDict()

        counter = 0
        for element in self.__iter__():
            if 'term' not in element or '@id' not in element['term']:
                items[f"no-term-no_id-{counter}"] = element
                counter += 1
            elif element['term']['@id'] not in items:
                items[element['term']['@id']] = element
            elif isinstance(items[element['term']['@id']], list):
                items[element['term']['@id']].append(element)
            else:
                items[element['term']['@id']] = [items[element['term']['@id']], element]

        return items
