import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.ch4N2ONh3AndNoxToAirCropResidueBurningAkagiEtAl2011AndIpcc2006\
    import CH4N2ONH3AndNOXToAirCropResidueBurningAkagiEtAl2011AndIpcc2006


class Test_ch4N2ONh3AndNoxToAirCropResidueBurningAkagiEtAl2011AndIpcc2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_ch4N2ONh3AndNoxToAirCropResidueBurningAkagiEtAl2011AndIpcc2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = CH4N2ONH3AndNOXToAirCropResidueBurningAkagiEtAl2011AndIpcc2006()

        engine.data.safe_representation['practices']['residueLeftOnField'] = {"term": {
                                                                        "@id": "residueLeftOnField",
                                                                        "@type": "Term",
                                                                        "name": "Residue left on field",
                                                                        "units": "percent",
                                                                        "definition": "The amount of crop residue left "
                                                                                      "on field. If the total crop "
                                                                                      "residue amount is inputted "
                                                                                      "separately as a Product, you "
                                                                                      "can also specify this as a "
                                                                                      "Destination for the residue.",
                                                                        "termType": "cropResidueManagement"
                                                                    },
                                                                    "value": 0,
                                                                    "@type": "Practice"
                                                                    }

        engine.data.safe_representation['practices']['residueBurnt'] = {"term": {
                                                                        "@id": "residueBurnt",
                                                                        "@type": "Term",
                                                                        "name": "Residue burnt",
                                                                        "units": "percent",
                                                                        "definition": "The amount of crop residue "
                                                                                      "burnt. If calculated, it should "
                                                                                      "be after multiplication by the "
                                                                                      "combustion factor (which allows "
                                                                                      "for residue burnt but not "
                                                                                      "combusted). If the total crop "
                                                                                      "residue amount is inputted "
                                                                                      "separately as a Product, you"
                                                                                      " can also specify this as a "
                                                                                      "Destination for the residue.",
                                                                        "termType": "cropResidueManagement"
                                                                        },
                                                                        "value": 0.9,
                                                                        "@type": "Practice"
                                                                        }

        engine.data.safe_representation['products']['aboveGroundCropResidueDryMatter'] = {
            "term": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "aboveGroundCropResidueDryMatter",
                "@type": "Term",
                "name": "Above ground crop residue (dry matter)",
                "units": "kg dry matter",
                "definition": "The total amount of above ground crop residue as dry matter. Can be partitioned into "
                              "different destinations (e.g. burning, left on field, etc.). Properties can be added, "
                              "such as the nitrogen composition.",
                "defaultProperties": [
                    {
                        "term": {
                            "@context": "https://www-staging.hestia.earth/Term.jsonld",
                            "@id": "landUseType",
                            "@type": "Term",
                            "name": "Land Use Type",
                            "definition": "The broad land use type for the given product.",
                            "termType": "property"
                        },
                        "value": "Agriculture",
                        "@type": "Property"
                    }
                ],
                "termType": "crop"
            },
            "value": [
                364.5
            ],
            "methodDescription": "IPCC (2006) standard methodology",
            "destination": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "burning",
                "@type": "Term",
                "name": "Left on field",
                "definition": "The product is left on the field.",
                "termType": "destination"
            },
            "@type": "Product"
        }

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.ch4ToAirCropResidueBurning_expected = 1.9092510000000003
        self.n2OToAirCropResidueBurningDirect_expected = 0.022963500000000005
        self.nh3ToAirCropResidueBurning_expected = 0.7118685
        self.noxToAirCropResidueBurning_expected = 1.0202354999999999

    def test_check_ch4ToAirCropResidueBurning(self):

        check_computed = self.model.check_ch4ToAirCropResidueBurning(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirCropResidueBurningDirect(self):

        check_computed = self.model.check_n2OToAirCropResidueBurningDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_nh3ToAirCropResidueBurning(self):

        check_computed = self.model.check_nh3ToAirCropResidueBurning(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_noxToAirCropResidueBurning(self):

        check_computed = self.model.check_noxToAirCropResidueBurning(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_ch4ToAirCropResidueBurning(self):

        self.model.check_ch4ToAirCropResidueBurning(self.representation)
        ch4ToAirCropResidueBurning_computed = \
            self.model.calculate_ch4ToAirCropResidueBurning()

        self.assertEqual(ch4ToAirCropResidueBurning_computed, self.ch4ToAirCropResidueBurning_expected)

    def test_n2OToAirCropResidueBurningDirect(self):

        self.model.check_n2OToAirCropResidueBurningDirect(self.representation)
        n2OToAirCropResidueBurningDirect_computed = \
            self.model.calculate_n2OToAirCropResidueBurningDirect()

        self.assertEqual(n2OToAirCropResidueBurningDirect_computed, self.n2OToAirCropResidueBurningDirect_expected)

    def test_nh3ToAirCropResidueBurning(self):

        self.model.check_nh3ToAirCropResidueBurning(self.representation)
        nh3ToAirCropResidueBurning_computed = \
            self.model.calculate_nh3ToAirCropResidueBurning()

        self.assertEqual(nh3ToAirCropResidueBurning_computed, self.nh3ToAirCropResidueBurning_expected)

    def test_noxToAirCropResidueBurning(self):

        self.model.check_noxToAirCropResidueBurning(self.representation)
        noxToAirCropResidueBurning_computed = \
            self.model.calculate_noxToAirCropResidueBurning()

        self.assertEqual(noxToAirCropResidueBurning_computed, self.noxToAirCropResidueBurning_expected)


if __name__ == '__main__':
    unittest.main()
