import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.co2ToAirFuelCombustionBiophysical import CO2ToAirFuelCombustionBiophysical


class Test_CO2ToAirFuelCombustionBiophysical(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_CO2ToAirFuelCombustionBiophysical, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = CO2ToAirFuelCombustionBiophysical()

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.co2ToAirFuelCombustion_expected = 57.52916

    def test_check_co2ToAirFuelCombustion(self):

        check_computed = self.model.check_co2ToAirFuelCombustion(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_co2ToAirFuelCombustion(self):

        self.model.check_co2ToAirFuelCombustion(self.representation)
        co2ToAirFuelCombustion_computed = \
            self.model.calculate_co2ToAirFuelCombustion()

        self.assertEqual(co2ToAirFuelCombustion_computed, self.co2ToAirFuelCombustion_expected)


if __name__ == '__main__':
    unittest.main()
