import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.co2ToAirLimeAndUreaIpcc2006\
    import CO2ToAirLimeAndUreaIpcc2006


class Test_co2ToAirLimeAndUreaIpcc2006(unittest.TestCase):

    def __init__(self, *args, **kwargs):

        super(Test_co2ToAirLimeAndUreaIpcc2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        self.model = CO2ToAirLimeAndUreaIpcc2006()

        engine.import_data(cycle, site)

        engine.data.safe_representation['inputs']['lime'] =\
            {'term': {'termType': "soilAmendment", '@id': "lime"},
             'value': [0]}

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.co2ToAirLimeHydrolysis_expected = 0.0
        self.co2ToAirUreaHydrolysis_expected = 109.97001499250376

    def test_check_co2ToAirLimeHydrolysis(self):

        check_computed = self.model.check_co2ToAirLimeHydrolysis(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_co2ToAirUreaHydrolysis(self):

        check_computed = self.model.check_co2ToAirUreaHydrolysis(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_calculate_co2ToAirLimeHydrolysis(self):

        self.model.check_co2ToAirLimeHydrolysis(self.representation)
        co2ToAirLimeHydrolysis_coumputed = self.model.calculate_co2ToAirLimeHydrolysis()

        self.assertEqual(co2ToAirLimeHydrolysis_coumputed, self.co2ToAirLimeHydrolysis_expected)

    def test_calculate_co2ToAirUreaHydrolysis(self):

        self.model.check_co2ToAirUreaHydrolysis(self.representation)
        co2ToAirUreaHydrolysis_computed = self.model.calculate_co2ToAirUreaHydrolysis()

        self.assertEqual(co2ToAirUreaHydrolysis_computed, self.co2ToAirUreaHydrolysis_expected)


if __name__ == '__main__':
    unittest.main()
