import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.n2OToAirFertilizerAndExcretaDirectIpcc2006\
    import N2OToAirFertilizerAndExcretaDirectIpcc2006


class Test_n2OToAirFertilizerAndExcretaDirectIpcc2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_n2OToAirFertilizerAndExcretaDirectIpcc2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        self.model = N2OToAirFertilizerAndExcretaDirectIpcc2006()

        engine.import_data(cycle, site)

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.n2OToAirInorganicFertilizerDirect_expected = 1.0997751124437782
        self.n2OToAirOrganicFertilizerDirect_expected = 0.0

    def test_check_n2OToAirInorganicFertilizerDirect(self):

        check_computed = self.model.check_n2OToAirInorganicFertilizerDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirOrganicFertilizerDirect(self):

        check_computed = self.model.check_n2OToAirOrganicFertilizerDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_calculate_n2OToAirInorganicFertilizerDirect(self):

        self.model.check_n2OToAirInorganicFertilizerDirect(self.representation)
        n2OToAirInorganicFertilizerDirect_computed =\
            self.model.calculate_n2OToAirInorganicFertilizerDirect()

        self.assertEqual(n2OToAirInorganicFertilizerDirect_computed, self.n2OToAirInorganicFertilizerDirect_expected)

    def test_calculate_n2OToAirOrganicFertilizerDirect(self):

        self.model.check_n2OToAirOrganicFertilizerDirect(self.representation)
        n2OToAirOrganicFertilizerDirect_computed\
            = self.model.calculate_n2OToAirOrganicFertilizerDirect()

        self.assertEqual(n2OToAirOrganicFertilizerDirect_computed, self.n2OToAirOrganicFertilizerDirect_expected)


if __name__ == '__main__':
    unittest.main()
