import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.n2OToAirFertilizerAndExcretaIndirectIpcc2006 import\
    N2OToAirFertilizerAndExcretaIndirectIpcc2006


class Test_n2OToAirFertilizerAndExcretaIndirectIpcc2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_n2OToAirFertilizerAndExcretaIndirectIpcc2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Cycle/Vgzlsfi1bT_.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Site/8-gSWoPHEQx.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        engine.data.safe_representation['emissions']['nh3ToAirInorganicFertilizer'] = {
            'term': {
                'name': "nh3ToAirInorganicFertilizer",
                '@id': "nh3ToAirInorganicFertilizer"
            },
            'value': [15.9]
        }

        engine.data.safe_representation['emissions']['noxToAirInorganicFertilizer'] = {
            'term': {
                'name': "noxToAirInorganicFertilizer",
                '@id': "noxToAirInorganicFertilizer"
            },
            'value': [5.35553651745556]

        }

        engine.data.safe_representation['emissions']['no3ToGroundwaterInorganicFertilizer'] = {
            'term': {
                'name': "no3ToGroundwaterInorganicFertilizer",
                '@id': "no3ToGroundwaterInorganicFertilizer"
            },
            'value': [101.812807881773]
        }

        engine.data.safe_representation['emissions']['nh3ToAirOrganicFertilizer'] = {
            'term': {
                'name': "nh3ToAirOrganicFertilizer",
                '@id': "nh3ToAirOrganicFertilizer"
            },
            'value': [0.0]
        }

        engine.data.safe_representation['emissions']['noxToAirOrganicFertilizer'] = {
            'term': {
                'name': "noxToAirOrganicFertilizer",
                '@id': "noxToAirOrganicFertilizer"
            },
            'value': [100.0]

        }

        engine.data.safe_representation['emissions']['no3ToGroundwaterOrganicFertilizer'] = {
            'term': {
                'name': "no3ToGroundwaterOrganicFertilizer",
                '@id': "no3ToGroundwaterOrganicFertilizer"
            },
            'value': [100.0]
        }

        engine.data.safe_representation['emissions']['nh3ToAirExcreta'] = {
            'term': {
                'name': "nh3ToAirExcreta",
                '@id': "nh3ToAirExcreta"
            },
            'value': [0.0]
        }

        engine.data.safe_representation['emissions']['noxToAirExcreta'] = {
            'term': {
                'name': "noxToAirExcreta",
                '@id': "noxToAirExcreta"
            },
            'value': [100.0]

        }

        engine.data.safe_representation['emissions']['no3ToGroundwaterExcreta'] = {
            'term': {
                'name': "no3ToGroundwaterExcreta",
                '@id': "no3ToGroundwaterExcreta"
            },
            'value': [100.0]
        }

        engine.data.safe_representation['emissions']['nh3ToAirCropResidueDecomposition'] = {
            'term': {
                'name': "nh3ToAirCropResidueDecomposition",
                '@id': "nh3ToAirCropResidueDecomposition"
            },
            'value': [50.0]
        }

        engine.data.safe_representation['emissions']['noxToAirCropResidueDecomposition'] = {
            'term': {
                'name': "noxToAirCropResidueDecomposition",
                '@id': "noxToAirCropResidueDecomposition"
            },
            'value': [0.67232275897729]

        }

        engine.data.safe_representation['emissions']['no3ToGroundwaterCropResidueDecomposition'] = {
            'term': {
                'name': "no3ToGroundwaterCropResidueDecomposition",
                '@id': "no3ToGroundwaterCropResidueDecomposition"
            },
            'value': [12.7813651669058]
        }

        self.model = N2OToAirFertilizerAndExcretaIndirectIpcc2006()

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.n2OToAirInorganicFertilizerIndirect_expected = 0.5157445467639602
        self.n2OToAirOrganicFertilizerIndirect_expected = 0.9995938070466025
        self.n2OToAirExcretaIndirect_expected = 0.9995938070466025
        self.n2OToAirCropResidueDecompositionIndirect_expected = 0.6850254846497007

    def test_check_n2OToAirInorganicFertilizerIndirect(self):

        check_computed = self.model.check_n2OToAirInorganicFertilizerIndirect(self.representation)

        self.assertEqual(self.check_expected, check_computed)

    def test_n2OToAirInorganicFertilizerIndirect(self):

        self.model.check_n2OToAirInorganicFertilizerIndirect(self.representation)
        n2OToAirInorganicFertilizerIndirect_computed = self.model.calculate_n2OToAirInorganicFertilizerIndirect()

        self.assertEqual(self.n2OToAirInorganicFertilizerIndirect_expected,
                         n2OToAirInorganicFertilizerIndirect_computed)

    def test_n2OToAirOrganicFertilizerIndirect(self):

        self.model.check_n2OToAirOrganicFertilizerIndirect(self.representation)
        n2OToAirOrganicFertilizerIndirect_computed = self.model.calculate_n2OToAirOrganicFertilizerIndirect()

        self.assertEqual(self.n2OToAirOrganicFertilizerIndirect_expected, n2OToAirOrganicFertilizerIndirect_computed)

    def test_n2OToAirExcretaIndirect(self):

        self.model.check_n2OToAirExcretaIndirect(self.representation)
        n2OToAirExcretaIndirect_computed = self.model.calculate_n2OToAirExcretaIndirect()

        self.assertEqual(self.n2OToAirExcretaIndirect_expected, n2OToAirExcretaIndirect_computed)

    def test_n2OToAirCropResidueDecompositionIndirect(self):

        self.model.check_n2OToAirCropResidueDecompositionIndirect(self.representation)
        n2OToAirCropResidueDecompositionIndirect_computed =\
            self.model.calculate_n2OToAirCropResidueDecompositionIndirect()

        self.assertEqual(self.n2OToAirCropResidueDecompositionIndirect_expected,
                         n2OToAirCropResidueDecompositionIndirect_computed)


if __name__ == '__main__':
    unittest.main()
