import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.n2OToAirFertilizerAndExcretadirectStehfestBouwman2006\
    import N2OToAirFertilizerAndExcretadirectStehfestBouwman2006


class Test_n2OToAirFertilizerAndExcretadirectStehfestBouwman2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_n2OToAirFertilizerAndExcretadirectStehfestBouwman2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        engine.data.safe_representation['products']['aboveGroundCropResidueDryMatter'] = {
            "term": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "aboveGroundCropResidueDryMatter",
                "@type": "Term",
                "name": "Above ground crop residue (dry matter)",
                "units": "kg dry matter",
                "definition": "The total amount of above ground crop residue as dry matter. Can be partitioned into "
                              "different destinations (e.g. burning, left on field, etc.). Properties can be added, "
                              "such as the nitrogen composition.",
                "termType": "crop"
            },
            "value": [
                0
            ],
            "methodDescription": "IPCC (2006) standard methodology",
            "destination": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "leftonfield",
                "@type": "Term",
                "name": "Left on field",
                "definition": "The product is left on the field.",
                "termType": "destination"
            },
            "@type": "Product",
            "properties": {
                "nitrogenContent": {
                    "term": {
                        "@id": "nitrogenContent",
                        "@type": "Term"
                    },
                    "value": "0.007",
                    "@type": "Property"
                }
            }
        }

        self.model = N2OToAirFertilizerAndExcretadirectStehfestBouwman2006()
        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.n2OToAirAllOriginsDirect_expected = 2.0382021488243436
        self.n2OToAirInorganicFertilizerDirect_expected = 2.0382021488243436
        self.n2OToAirOrganicFertilizerDirect_expected = 0.0
        self.n2OToAirExcretaDirect_expected = 0.0
        self.n2OToAirCropResidueDecompositionDirect_expected = 0.0

    def test_check_n2OToAirAllOriginsDirect(self):

        check_computed = self.model.check_n2OToAirAllOriginsDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirInorganicFertilizerDirect(self):

        check_computed = self.model.check_n2OToAirInorganicFertilizerDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirOrganicFertilizerDirect(self):

        check_computed = self.model.check_n2OToAirOrganicFertilizerDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirExcretaDirect(self):

        check_computed = self.model.check_n2OToAirExcretaDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_check_n2OToAirCropResidueDecompositionDirect(self):

        check_computed = self.model.check_n2OToAirCropResidueDecompositionDirect(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_n2OToAirAllOriginsDirect(self):

        self.model.check_n2OToAirAllOriginsDirect(self.representation)
        n2OToAirAllOriginsDirect_computed = self.model.calculate_n2OToAirAllOriginsDirect()

        self.assertEqual(n2OToAirAllOriginsDirect_computed, self.n2OToAirAllOriginsDirect_expected)

    def test_n2OToAirInorganicFertilizerDirect(self):

        self.model.check_n2OToAirInorganicFertilizerDirect(self.representation)
        n2OToAirInorganicFertilizerDirect_computed = \
            self.model.calculate_n2OToAirInorganicFertilizerDirect()

        self.assertEqual(n2OToAirInorganicFertilizerDirect_computed, self.n2OToAirInorganicFertilizerDirect_expected)

    def test_n2OToAirOrganicFertilizerDirect(self):

        self.model.check_n2OToAirOrganicFertilizerDirect(self.representation)
        n2OToAirOrganicFertilizerDirect_computed = \
            self.model.calculate_n2OToAirOrganicFertilizerDirect()

        self.assertEqual(n2OToAirOrganicFertilizerDirect_computed, self.n2OToAirOrganicFertilizerDirect_expected)

    def test_n2OToAirExcretaDirect(self):

        self.model.check_n2OToAirExcretaDirect(self.representation)
        n2OToAirExcretaDirect_computed = self.model.calculate_n2OToAirExcretaDirect()

        self.assertEqual(n2OToAirExcretaDirect_computed, self.n2OToAirExcretaDirect_expected)

    def test_n2OToAirCropResidueDecompositionDirect(self):

        self.model.check_n2OToAirCropResidueDecompositionDirect(self.representation)
        n2OToAirCropResidueDecompositionDirect_computed = \
            self.model.calculate_n2OToAirCropResidueDecompositionDirect()

        self.assertEqual(n2OToAirCropResidueDecompositionDirect_computed,
                         self.n2OToAirCropResidueDecompositionDirect_expected)


if __name__ == '__main__':
    unittest.main()
