import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.nh3ToAirCropResidueDecompositionDeRuijterEtAl2010\
    import NH3ToAirCropResidueDecompositionDeRuijterEtAl2010


class Test_nh3ToAirCropResidueDecompositionDeRuijterEtAl2010(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_nh3ToAirCropResidueDecompositionDeRuijterEtAl2010, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = NH3ToAirCropResidueDecompositionDeRuijterEtAl2010()

        ersr = engine.data.safe_representation
        ersr['products']['aboveGroundCropResidueDryMatter'] = {}
        ersr['products']['aboveGroundCropResidueDryMatter']['value'] = [10]
        ersr['products']['aboveGroundCropResidueDryMatter']['properties'] = {}
        ersr['products']['aboveGroundCropResidueDryMatter']['properties']['nitrogenContent'] = {}
        final = ersr['products']['aboveGroundCropResidueDryMatter']['properties']['nitrogenContent']
        final['value'] = 5

        self.representation = ersr

        self.check_expected = True
        self.nh3ToAirCropResidueDecomposition_expected = 0.19794689199999999

    def test_check_nh3ToAirCropResidueDecomposition(self):

        check_computed = self.model.check_nh3ToAirCropResidueDecomposition(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_nh3ToAirCropResidueDecomposition(self):

        self.model.check_nh3ToAirCropResidueDecomposition(self.representation)
        nh3ToAirCropResidueDecomposition_computed = \
            self.model.calculate_nh3ToAirCropResidueDecomposition()

        self.assertEqual(nh3ToAirCropResidueDecomposition_computed, self.nh3ToAirCropResidueDecomposition_expected)


if __name__ == '__main__':
    unittest.main()
