import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.nh3ToAirInorganicFertilizerEuropeanEnvironmentAgency2016\
    import NH3ToAirInorganicFertilizerEuropeanEnvironmentAgency2016


class Test_nh3ToAirInorganicFertilizerEuropeanEnvironmentAgency2016(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_nh3ToAirInorganicFertilizerEuropeanEnvironmentAgency2016, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = NH3ToAirInorganicFertilizerEuropeanEnvironmentAgency2016()

        engine.data.safe_representation['inputs']['ureaAsN'] = {'value': [70]}
        engine.data.safe_representation['inputs']['ureaAmmoniumSulphateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['ammoniumBicarbonateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['ammoniumSulphateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['ureaAmmoniumNitrateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['ammoniumNitrateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['ammoniumChlorideAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['potassiumNitrateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['potassiumNitrateAsK2O'] = {'value': [0]}
        engine.data.safe_representation['inputs']['calciumAmmoniumNitrateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['anhydrousAmmoniumAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['diammoniumPhosphateAsN'] = {'value': [0]}
        engine.data.safe_representation['inputs']['monoammoniumPhosphateAsP2O5'] = {'value': [0]}

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.nh3ToAirInorganicFertilizer_expected = 11.13

    def test_check_nh3ToAirInorganicFertilizer(self):

        check_computed = self.model.check_nh3ToAirInorganicFertilizer(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_nh3ToAirInorganicFertilizer(self):

        self.model.check_nh3ToAirInorganicFertilizer(self.representation)
        nh3ToAirInorganicFertilizer_computed = \
            self.model.calculate_nh3ToAirInorganicFertilizer()

        self.assertEqual(nh3ToAirInorganicFertilizer_computed, self.nh3ToAirInorganicFertilizer_expected)


if __name__ == '__main__':
    unittest.main()
