import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.utils import EngineDict
from hestia_earth.calculation.models.no3ToGroundwaterAllInputsPooreNemecek2018\
    import NO3ToGroundwaterAllInputsPooreNemecek2018


class Test_no3ToGroundwaterAllInputsPooreNemecek2018(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_no3ToGroundwaterAllInputsPooreNemecek2018, self).__init__(*args, **kwargs)

        with open(f'{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld') as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f'{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld') as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        self.model = NO3ToGroundwaterAllInputsPooreNemecek2018()

        engine.import_data(cycle, site)

        engine.data.representation_cycle['site']['measurements']['annualRainfallYearOfCycle'] =\
            EngineDict({'value': [10.0]})
        engine.data.representation_cycle['inputs']['compostAsN'] =\
            EngineDict({'term': {'@id': 'compostAsN',
                                 'units': 'kg N',
                                 'termType': 'organicFertilizer'},
                        'value': [0]})
        engine.data.representation_cycle['inputs']['beefCattleSolidManureDryAsN'] =\
            EngineDict({'term': {'@id': 'compostAsN',
                                 'units': 'kg N',
                                 'termType': 'animalProduct'},
                        'value': [0]})

        abg_node = {'term': {
                        '@id': 'aboveGroundCropResidueDryMatter',
                        'units': 'kg dry matter',
                        'termType': 'crop'},
                    'destination': {
                        '@context': "https://www-staging.hestia.earth/Term.jsonld",
                        '@id': 'leftOnField',
                        '@type': 'Term',
                        'name': 'Left on field',
                        'definition': "The product is left on the field.",
                        'termType': 'destination'
                    },
                    'properties': {
                        'nitrogenContent': {
                            'term': {
                             '@id': 'nitrogenContent',
                             '@type': 'Term'
                            },
                            'value': '0'
                        }
                    },
                    'value': [0]
                    }
        engine.data.representation_cycle['products']['aboveGroundCropResidueDryMatter'] = EngineDict(abg_node)

        self.representation = engine.data.representation_cycle

        self.check_expected = True
        self.check_leaching_expected = 0.067
        self.no3ToGroundwaterAllOrigins_expected = 20.760959520239883
        self.no3ToGroundwaterOrganicFertilizer_expected = 0.0
        self.no3ToGroundwaterInorganicFertilizer_expected = 20.760959520239883
        self.no3ToGroundwaterExcreta_expected = 0.0
        self.no3ToGroundwaterCropResidueDecomposition_expected = 0.0

    def test_check(self):

        check_computed = self.model.check_no3ToGroundwaterAllOrigins(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_get_leaching_factor(self):

        self.model.check_no3ToGroundwaterAllOrigins(self.representation)
        check_leaching_computed = self.model.get_leaching_factor()

        self.assertEqual(check_leaching_computed, self.check_leaching_expected)

    def test_no3ToGroundwaterAllOrigins(self):

        self.model.check_no3ToGroundwaterAllOrigins(self.representation)
        no3ToGroundwaterAllOrigins_computed = self.model.calculate_no3ToGroundwaterAllOrigins()

        self.assertEqual(no3ToGroundwaterAllOrigins_computed, self.no3ToGroundwaterAllOrigins_expected)

    def test_no3ToGroundwaterOrganicFertilizer(self):

        self.model.check_no3ToGroundwaterOrganicFertilizer(self.representation)
        no3ToGroundwaterOrganicFertilizer_computed =\
            self.model.calculate_no3ToGroundwaterOrganicFertilizer()

        self.assertEqual(no3ToGroundwaterOrganicFertilizer_computed, self.no3ToGroundwaterOrganicFertilizer_expected)

    def test_no3ToGroundwaterInorganicFertilizer(self):

        self.model.check_no3ToGroundwaterInorganicFertilizer(self.representation)
        self.model.calculate_no3ToGroundwaterInorganicFertilizer()
        no3ToGroundwaterInorganicFertilizer_computed =\
            self.model.calculate_no3ToGroundwaterInorganicFertilizer()

        self.assertEqual(no3ToGroundwaterInorganicFertilizer_computed,
                         self.no3ToGroundwaterInorganicFertilizer_expected)

    def test_no3ToGroundwaterExcreta(self):

        self.model.check_no3ToGroundwaterExcreta(self.representation)
        no3ToGroundwaterExcreta_computed = self.model.calculate_no3ToGroundwaterExcreta()

        self.assertEqual(no3ToGroundwaterExcreta_computed, self.no3ToGroundwaterExcreta_expected)

    def test_no3ToGroundwaterCropResidueDecomposition(self):

        self.model.check_no3ToGroundwaterCropResidueDecomposition(self.representation)
        no3ToGroundwaterCropResidueDecomposition_computed = \
            self.model.calculate_no3ToGroundwaterCropResidueDecomposition()

        self.assertEqual(no3ToGroundwaterCropResidueDecomposition_computed,
                         self.no3ToGroundwaterCropResidueDecomposition_expected)


if __name__ == '__main__':
    unittest.main()
