import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.noxToAirFertilizerAndExcretaStehfestBouwman2006Tier1\
    import NOXToAirFertilizerAndExcretaStehfestBouwman2006Tier1


class Test_noxToAirFertilizerAndExcretaStehfestBouwman2006Tier1(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_noxToAirFertilizerAndExcretaStehfestBouwman2006Tier1, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = NOXToAirFertilizerAndExcretaStehfestBouwman2006Tier1()

        engine.data.safe_representation['products']['aboveGroundCropResidueDryMatter'] = {
            "term": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "aboveGroundCropResidueDryMatter",
                "@type": "Term",
                "name": "Above ground crop residue (dry matter)",
                "units": "kg dry matter",
                "definition": "The total amount of above ground crop residue as dry matter. Can be partitioned into "
                              "different destinations (e.g. burning, left on field, etc.). Properties can be added, "
                              "such as the nitrogen composition.",
                "defaultProperties": [
                    {
                        "term": {
                            "@context": "https://www-staging.hestia.earth/Term.jsonld",
                            "@id": "landUseType",
                            "@type": "Term",
                            "name": "Land Use Type",
                            "definition": "The broad land use type for the given product.",
                            "termType": "property"
                        },
                        "value": "Agriculture",
                        "@type": "Property"
                    }
                ],
                "termType": "crop"
            },
            "value": [
                364.5
            ],
            "methodDescription": "IPCC (2006) standard methodology",
            "destination": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "burning",
                "@type": "Term",
                "name": "Left on field",
                "definition": "The product is left on the field.",
                "termType": "destination"
            },
            "@type": "Product",
            "properties": {
                "nitrogenContent": {
                    "term": {
                        "@id": "nitrogenContent",
                        "@type": "Term"
                    },
                    "value": "0.007",
                    "@type": "Property"
                }
            }
        }

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.noxToAirAllOrigins_expected = 0.827190993748

    def test_check_noxToAirAllOrigins(self):

        check_computed = self.model.check_noxToAirAllOrigins(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_noxToAirAllOrigins(self):

        self.model.check_noxToAirAllOrigins(self.representation)
        noxToAirAllOrigins_computed = \
            self.model.calculate_noxToAirAllOrigins()

        self.assertEqual(noxToAirAllOrigins_computed, self.noxToAirAllOrigins_expected)


if __name__ == '__main__':
    unittest.main()
