import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.pErosionSalcaSchererPfister2015 import PErosionSalcaSchererPfister2015


class Test_pErosionSalcaSchererPfister2015(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_pErosionSalcaSchererPfister2015, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Cycle/Vgzlsfi1bT_.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Site/8-gSWoPHEQx.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        engine.data.safe_representation['practices']['practiceFactor'] = {
            "term": {
                "@context": "https://www-staging.hestia.earth/Term.jsonld",
                "@id": "practiceFactor",
                "@type": "Term",
                "name": "Practice Factor",
                "units": "Number",
                "definition": "Practice factor",
                "termType": "landUseManagement"
            },
            "value": 0.945,
            "@type": "Practice"
        }

        self.model = PErosionSalcaSchererPfister2015()

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.pErosionAllOrigins_expected = 24.6192729236845

    def test_check_pErosionAllOrigins(self):

        check_computed = self.model.check_pErosionAllOrigins(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_pErosionAllOrigins(self):

        self.model.check_pErosionAllOrigins(self.representation)
        pErosionAllOrigins_computed = self.model.calculate_pErosionAllOrigins()

        self.assertEqual(pErosionAllOrigins_computed, self.pErosionAllOrigins_expected)


if __name__ == '__main__':
    unittest.main()
