import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.pToDrainageWaterSalcaPrasuhn2006 import PToDrainageWaterSalcaPrasuhn2006


class Test_pToDrainageWaterSalcaPrasuhn2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_pToDrainageWaterSalcaPrasuhn2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Cycle/Vgzlsfi1bT_.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Site/8-gSWoPHEQx.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        engine.data.safe_representation['emissions']['pToSurfacewaterAllInputs'] = {
            'term': {
                'name': "pToSurfacewaterAllInputs",
                '@id': "pToSurfacewaterAllInputs"
            },
            'value': [0.0]
        }

        self.model = PToDrainageWaterSalcaPrasuhn2006()

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.pToDrainageWaterAllInputs_expected = 0.42000000000000004

    def test_check_pToDrainageWaterAllInputs(self):

        check_computed = self.model.check_pToDrainageWaterAllInputs(self.representation)

        self.assertEqual(self.check_expected, check_computed)

    def test_pToDrainageWaterAllInputs(self):

        self.model.check_pToDrainageWaterAllInputs(self.representation)
        pToDrainageWaterAllInputs_computed = self.model.calculate_pToDrainageWaterAllInputs()

        self.assertEqual(self.pToDrainageWaterAllInputs_expected, pToDrainageWaterAllInputs_computed)


if __name__ == '__main__':
    unittest.main()
