import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.models.pToGroundwaterSalcaPrasuhn2006 import PToGroundwaterSalcaPrasuhn2006


class Test_pToGroundwaterSalcaPrasuhn2006(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super(Test_pToGroundwaterSalcaPrasuhn2006, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Cycle/Vgzlsfi1bT_.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/Vgzlsfi1bT_/Site/8-gSWoPHEQx.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        engine.import_data(cycle, site)

        self.model = PToGroundwaterSalcaPrasuhn2006()

        self.representation = engine.data.safe_representation

        self.check_expected = True
        self.pToGroundwaterAllInputs_expected = 0.0

    def test_check_pToGroundwaterAllInputs(self):

        check_computed = self.model.check_pToGroundwaterAllInputs(self.representation)

        self.assertEqual(check_computed, self.check_expected)

    def test_pToGroundwaterAllInputs(self):

        self.model.check_pToGroundwaterAllInputs(self.representation)
        pToGroundwaterAllInputs_computed = self.model.calculate_pToGroundwaterAllInputs()

        self.assertEqual(pToGroundwaterAllInputs_computed, self.pToGroundwaterAllInputs_expected)


if __name__ == '__main__':
    unittest.main()
