import unittest
import json
from tests.utils import fixtures_path
from hestia_earth.calculation.abstract_engine import Engine
from hestia_earth.calculation.emissionsResourceUses.landOccupation\
    import LandOccupation


class Test_landOccupation(unittest.TestCase):

    def __init__(self, *args, **kwargs):

        # generic crop grain example
        super(Test_landOccupation, self).__init__(*args, **kwargs)

        with open(f"{fixtures_path}/WI5hWtsqlu/Cycle/WI5hWtsqlu.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        with open(f"{fixtures_path}/WI5hWtsqlu/Site/2ppyB9xfAo.jsonld") as jsonld_file:
            site = json.load(jsonld_file)

        engine = Engine()

        self.model = LandOccupation()

        engine.import_data(cycle, site)

        self.representation_cycle = engine.data.representation_cycle

        self.landOccupation_expected = {'landOccupation': 3.758573388203018}

    def test_calculate_landOccupation(self):

        landOccupation_coumputed = self.model.calculate(self.representation_cycle,
                                                        self.representation_cycle['products']['cerealsGrain'])

        self.assertEqual(self.landOccupation_expected, landOccupation_coumputed)

    def test_hazelnuts(self):
        # Iranian Hazelnut example
        with open(f"{fixtures_path}/landOccupation/hazelnuts_example.jsonld") as jsonld_file:
            cycle = json.load(jsonld_file)

        engine = Engine()

        self.model = LandOccupation()

        engine.import_data(cycle=cycle)

        self.representation_cycle = engine.data.representation_cycle

        self.landOccupation_HZ_expected = {'landOccupation': 22.21235006663705}

        landOccupation_coumputed = self.model.calculate(self.representation_cycle,
                                                        self.representation_cycle['products']['hazelnutInShell'])

        self.assertEqual(self.landOccupation_HZ_expected, landOccupation_coumputed)
    # TODO: add new tests, 1) multiple examples that hit all if statement possibilities
    #  2) add another successful run, different product, ideally with cycle duration + practice data.
    # TODO: Update hazelnut result when orchard code added


if __name__ == '__main__':
    unittest.main()
